@@ -10,8 +10,8 @@ cluster_execute <- function(first,
                             update_switch = FALSE) {
 
   # Capture current working directory and reset it on function exit
-  oldwd <- getwd()
-  on.exit(setwd(oldwd))
+  ..oldwd <- getwd()
+  on.exit(setwd(..oldwd))
 
   # error handle invalid options
   handle_errors(keep_errors, "is.boolean")
@@ -29,19 +29,23 @@ cluster_execute <- function(first,
   rm(last)
   rm(cluster_config)
 
+  # Create a new environment to evaluate code blocks within and get a reference
+  # to the calling environment
+  ..env_cl <- new.env()
+  ..env_calling <- parent.frame(n=2)
+
   # Run all code locally if simulation is not being run on cluster
-  # !!!!! TO-DO make this work for update_switch = TRUE
+  # !!!!! TO-DO make sure this works for update_switch = TRUE
   if (Sys.getenv("sim_run")=="") {
 
-    # Run code locally (`first` block)
-    eval(..first)
+    # Run code (`first` block)
+    eval(..first, envir=..env_cl)
 
     # Extract the simulation object variable name
-    ..env <- environment()
     ..count <- 0
     ..sim_var <- NA
-    for (obj_name in ls(..env)) {
-      if ("sim_obj" %in% class(get(x=obj_name, envir=..env))) {
+    for (obj_name in ls(..env_cl)) {
+      if (methods::is(get(x=obj_name, envir=..env_cl), "sim_obj")) {
         ..sim_var <- obj_name
         ..count <- ..count + 1
       }
@@ -54,16 +58,53 @@ cluster_execute <- function(first,
                  "created in the `first` block"))
     }
     rm(..count)
-    rm(..env)
 
-    # Assign simulation object to ..sim_var in the parent environment
-    assign(x=..sim_var, value=eval(as.name(..sim_var)), envir=parent.frame(n=2))
+    # Add objects in the calling environment to the simulation object
+    # !!!!! Functionize this later
+    for (obj_name in ls(..env_calling)) {
+      obj <- get(x=obj_name, envir=..env_calling)
+      if (!methods::is(obj,"sim_obj") && obj_name!="L") {
+        assign(x=obj_name, value=obj,
+               envir=get(..sim_var, envir=..env_cl)$vars$env)
+      }
+    }
+
+    # Save a copy of simulation object to the parent environment
+    assign(x = ..sim_var,
+           value = get(..sim_var, envir=..env_cl),
+           envir = ..env_calling)
 
     # Run code locally (`main` and `last` blocks)
-    eval(..main)
-    assign(x=..sim_var, value=eval(as.name(..sim_var)), envir=parent.frame(n=2))
-    eval(..last)
-    assign(x=..sim_var, value=eval(as.name(..sim_var)), envir=parent.frame(n=2))
+    eval(..main, envir=..env_cl)
+
+    # Add objects in the calling environment to the simulation object
+    # !!!!! Functionize this later
+    for (obj_name in ls(..env_calling)) {
+      obj <- get(x=obj_name, envir=..env_calling)
+      if (!methods::is(obj,"sim_obj") && obj_name!="L") {
+        assign(x=obj_name, value=obj,
+               envir=get(..sim_var, envir=..env_cl)$vars$env)
+      }
+    }
+
+    assign(x = ..sim_var,
+           value = get(..sim_var, envir=..env_cl),
+           envir = ..env_calling)
+    eval(..last, envir=..env_cl)
+
+    # Add objects in the calling environment to the simulation object
+    # !!!!! Functionize this later
+    for (obj_name in ls(..env_calling)) {
+      obj <- get(x=obj_name, envir=..env_calling)
+      if (!methods::is(obj,"sim_obj") && obj_name!="L") {
+        assign(x=obj_name, value=obj,
+               envir=get(..sim_var, envir=..env_cl)$vars$env)
+      }
+    }
+
+    assign(x = ..sim_var,
+           value = get(..sim_var, envir=..env_cl),
+           envir = ..env_calling)
 
   } else {
 
@@ -88,9 +129,9 @@ cluster_execute <- function(first,
     }
 
     # Error handling: test to see that we can write to cfg$dir
-    test_file <- paste0(..path_sim_obj, '.test')
+    ..test_file <- paste0(..path_sim_obj, '.test')
     tryCatch(
-      expr = { saveRDS(list(a=123,b=456), file=test_file) },
+      expr = { saveRDS(list(a=123,b=456), file=..test_file) },
       error = function(e) {
         stop(paste0("Directory ", ..cfg$dir, " is not writable."))
       }
@@ -98,7 +139,7 @@ cluster_execute <- function(first,
 
     # Error handling: test to see that we can read from cfg$dir
     tryCatch(
-      expr = { x <- readRDS(file=test_file) },
+      expr = { x <- readRDS(file=..test_file) },
       error = function(e) {
         stop(paste0("Directory ", ..cfg$dir, " is not readable."))
       }
@@ -106,14 +147,14 @@ cluster_execute <- function(first,
 
     # Error handling: test to see that we can delete from cfg$dir
     tryCatch(
-      expr = { unlink(test_file) },
+      expr = { unlink(..test_file) },
       error = function(e) {
         stop(paste0("Files cannot be deleted from directory ", ..cfg$dir, "."))
       }
     )
 
     # Set working directory
-    if (!is.null(..cfg$dir)) setwd(..cfg$dir)
+    if (!is.null(..cfg$dir)) { setwd(..cfg$dir) }
 
     # Remove old files
     if (dir.exists(..path_sim_res)) { unlink(..path_sim_res, recursive=TRUE) }
@@ -121,17 +162,15 @@ cluster_execute <- function(first,
     # Create directory to store simulation results
     dir.create(..path_sim_res)
 
-    ..start_time <- Sys.time()
-
     # Run 'first' code
-    eval(..first)
+    ..start_time <- Sys.time()
+    eval(..first, envir=..env_cl)
 
     # Extract the simulation object variable name
-    ..env <- environment()
     ..count <- 0
     ..sim_var <- NA
-    for (obj_name in ls(..env)) {
-      if ("sim_obj" %in% class(get(x=obj_name, envir=..env))) {
+    for (obj_name in ls(..env_cl)) {
+      if (methods::is(get(x=obj_name, envir=..env_cl),"sim_obj")) {
         ..sim_var <- obj_name
         ..count <- ..count + 1
       }
@@ -149,10 +188,20 @@ cluster_execute <- function(first,
                  "read in the `first` block"))
     }
     rm(..count)
-    rm(..env)
+
+    # Get reference to simulation object
+    ..sim <- get(..sim_var, envir=..env_cl)
+
+    # Add objects in the calling environment to the simulation object
+    # !!!!! Functionize this later
+    for (obj_name in ls(..env_calling)) {
+      obj <- get(x=obj_name, envir=..env_calling)
+      if (!methods::is(obj,"sim_obj") && obj_name!="L") {
+        assign(x=obj_name, value=obj, envir=..sim$vars$env)
+      }
+    }
 
     # Save simulation object
-    ..sim <- eval(as.name(..sim_var))
     ..sim$internals$sim_var <- ..sim_var
     ..sim$vars$start_time <- ..start_time
     ..sim$config$parallel <- "cluster"
@@ -161,7 +210,7 @@ cluster_execute <- function(first,
   } else if (Sys.getenv("sim_run") %in% c("main","last")) {
 
     # Set working directory
-    if (!is.null(..cfg$dir)) setwd(..cfg$dir)
+    if (!is.null(..cfg$dir)) { setwd(..cfg$dir) }
 
     tryCatch(
       ..sim <- readRDS(..path_sim_obj),
@@ -180,10 +229,10 @@ cluster_execute <- function(first,
   # MAIN: run simulation replicate and save results/errors
   if (Sys.getenv("sim_run")=="main") {
 
-    # if there are error files in the results directory and stop_at_error==TRUE
+    # if there are error files in the results directory and stop_at_error==TRUE,
     # skip this rep
     err_reps <- list.files(path = ..path_sim_res, pattern = "e_*")
-    if (!(length(err_reps) > 0 & ..sim$config$stop_at_error)){
+    if (!(length(err_reps) > 0 & ..sim$config$stop_at_error)) {
 
       # Error handling: tid_var and js
       if (is.null(..cfg$tid_var) && is.null(..cfg$js)) {
@@ -216,14 +265,10 @@ cluster_execute <- function(first,
 
       tid <- as.numeric(Sys.getenv(tid_var))
 
-      if (is.na(tid)) {
-        stop("Task ID is missing.")
-      }
+      if (is.na(tid)) { stop("Task ID is missing.") }
 
       add_to_tid <- as.numeric(Sys.getenv("sim_add_to_tid"))
-      if (!is.na(add_to_tid)) {
-        tid <- tid + add_to_tid
-      }
+      if (!is.na(add_to_tid)) { tid <- tid + add_to_tid }
 
       if (tid<1 || tid>..sim$vars$num_sim_total) {
         stop(paste(
@@ -231,21 +276,29 @@ cluster_execute <- function(first,
           ..sim$vars$num_sim_total
         ))
       } else {
-        if (update_switch) {
-          # for updating, need to add number of previously run sims
-          #     (num_sim_cuml)
-          tid <- tid + ..sim$internals$num_sim_cuml
-        }
-        # Run 'main' code
+        # For updating, need to add number of previously run sims
+        #     (num_sim_cuml)
+        if (update_switch) { tid <- tid + ..sim$internals$num_sim_cuml }
+
         ..sim$internals$tid <- tid
         rm(tid)
         rm(add_to_tid)
-        for (pkg in ..sim$config$packages) {
-          do.call("library", list(pkg))
+        for (pkg in ..sim$config$packages) { do.call("library", list(pkg)) }
+        assign(..sim$internals$sim_var, ..sim, envir=..env_cl)
+
+        # Run 'main' code
+        eval(..main, envir=..env_cl)
+        ..sim <- get(..sim$internals$sim_var, envir=..env_cl)
+
+        # Add objects in the calling environment to the simulation object
+        # !!!!! Functionize this later
+        for (obj_name in ls(..env_calling)) {
+          obj <- get(x=obj_name, envir=..env_calling)
+          if (!methods::is(obj,"sim_obj") && obj_name!="L") {
+            assign(x=obj_name, value=obj, envir=..sim$vars$env)
+          }
         }
-        assign(..sim$internals$sim_var, ..sim)
-        eval(..main)
-        assign("..sim", eval(as.name(..sim$internals$sim_var)))
+
       }
 
       # Parse results filename and save
@@ -283,7 +336,7 @@ cluster_execute <- function(first,
     # if there are error files in the results directory and stop_at_error==TRUE
     # skip this rep
     err_reps <- list.files(path = ..path_sim_res, pattern = "e_*")
-    if (length(err_reps) > 0 & ..sim$config$stop_at_error){
+    if (length(err_reps) > 0 & ..sim$config$stop_at_error) {
       unlink(paste0(..path_sim_res, "/r_*"))
     } else {
       # Process result/error files
@@ -316,7 +369,7 @@ cluster_execute <- function(first,
 
           e <- readRDS(paste0(..path_sim_res, "/", file))
 
-          if (class(e)=="data.frame") {
+          if (methods::is(e,"data.frame")) {
             if (is.null(errors_df)) {
               errors_df <- e
             } else {
@@ -329,7 +382,7 @@ cluster_execute <- function(first,
         } else if (substr(file,1,1) == "w") {
           w <- readRDS(paste0(..path_sim_res, "/", file))
 
-          if (class(w)=="data.frame") {
+          if (methods::is(w,"data.frame")) {
             if (is.null(warnings_df)) {
               warnings_df <- w
             } else {
@@ -339,9 +392,7 @@ cluster_execute <- function(first,
         }
       }
 
-      if (identical(results_complex,list())) {
-        results_complex <- NA
-      }
+      if (identical(results_complex,list())) { results_complex <- NA }
 
       if (update_switch) {
         # combine results and errors with existing results and errors
@@ -366,8 +417,7 @@ cluster_execute <- function(first,
       # Note: this code is somewhat redundant with the end of SimEngine::run
       if (!is.null(warnings_df)) {
         ..sim$warnings <- warnings_df
-      }
-      else {
+      } else {
         ..sim$warnings <- "No warnings"
       }
       if (!is.null(results_df) && !is.null(errors_df)) {
@@ -385,7 +435,7 @@ cluster_execute <- function(first,
         ..sim$errors <- errors_df
         ..sim$vars$run_state <- "run, all errors"
       } else {
-        stop("An unknown error occurred.")
+        stop("An unknown error occurred (CODE 102)")
       }
 
       levels_grid_big <- create_levels_grid_big(..sim)
@@ -393,7 +443,7 @@ cluster_execute <- function(first,
       if (update_switch) {
         prev_levels_grid_big <- ..sim$internals$levels_grid_big
 
-        # get levels / sim_ids that were previously run but are no longer needed
+        # Get levels / sim_ids that were previously run but are no longer needed
         extra_run <- dplyr::anti_join(
           prev_levels_grid_big[,-which(names(prev_levels_grid_big) %in%
                                          c("sim_uid", "level_id")),drop=F],
@@ -404,24 +454,24 @@ cluster_execute <- function(first,
                                         drop=F])
         )
 
-        # if keep_extra = FALSE, remove excess runs (from results, errors, and
+        # If keep_extra = FALSE, remove excess runs (from results, errors, and
         #     warnings)
-        if (!keep_extra & nrow(extra_run) > 0){
+        if (!keep_extra & nrow(extra_run) > 0) {
 
-          if (!is.character(..sim$results)){
+          if (!is.character(..sim$results)) {
             ..sim$results <- dplyr::anti_join(..sim$results,
-                                                  extra_run,
-                                                  by = names(extra_run))
+                                              extra_run,
+                                              by = names(extra_run))
           }
-          if (!is.character(..sim$errors)){
+          if (!is.character(..sim$errors)) {
             ..sim$errors <- dplyr::anti_join(..sim$errors,
-                                                 extra_run,
-                                                 by = names(extra_run))
+                                             extra_run,
+                                             by = names(extra_run))
           }
-          if (!is.character(..sim$warnings)){
+          if (!is.character(..sim$warnings)) {
             ..sim$warnings <- dplyr::anti_join(..sim$warnings,
-                                                   extra_run,
-                                                   by = names(extra_run))
+                                               extra_run,
+                                               by = names(extra_run))
           }
         }
       }
@@ -443,24 +493,30 @@ cluster_execute <- function(first,
       # This is done before running the 'last' code so that the compiled
       #   simulation object is saved even if there's an error with the 'last'
       #   code
-      for (i in 1:5){
+      # Note: the for-loop helps get around a bug related to file-locking
+      for (i in 1:5) {
         x <- unlink(..path_sim_res, recursive=TRUE)
-        if (x == 0){
-          break
-        }
+        if (x == 0) { break }
         Sys.sleep(0.2)
       }
       saveRDS(..sim, file=..path_sim_obj)
 
       # Run 'last' code
-      for (pkg in ..sim$config$packages) {
-        do.call("library", list(pkg))
+      for (pkg in ..sim$config$packages) { do.call("library", list(pkg)) }
+      assign(..sim$internals$sim_var, ..sim, envir=..env_cl)
+      eval(..last, envir=..env_cl)
+      ..sim <- get(..sim$internals$sim_var, envir=..env_cl)
+
+      # Add objects in the calling environment to the simulation object
+      # !!!!! Functionize this later
+      for (obj_name in ls(..env_calling)) {
+        obj <- get(x=obj_name, envir=..env_calling)
+        if (!methods::is(obj,"sim_obj") && obj_name!="L") {
+          assign(x=obj_name, value=obj, envir=..sim$vars$env)
+        }
       }
-      assign(..sim$internals$sim_var, ..sim)
-      eval(..last)
 
       # Save final simulation object (a second time, if 'last' code had no errors)
-      assign("..sim", eval(as.name(..sim$internals$sim_var)))
       ..sim$vars$end_time <- Sys.time()
       ..sim$vars$total_runtime <- as.numeric(
         difftime(..sim$vars$end_time, ..sim$vars$start_time),
