@@ -109,17 +109,16 @@
 #' # The following is a toy example of a simulation, illustrating the use of
 #' # the summarize function.
 #' sim <- new_sim()
-#' sim %<>% add_creator("create_data", function(n) { rpois(n, lambda=5) })
-#' sim %<>% add_method("estimator_1", function(dat) { mean(dat) })
-#' sim %<>% add_method("estimator_2", function(dat) { var(dat) })
-#' sim %<>% set_levels(
-#'   "n" = c(10, 100, 1000),
-#'   "estimator" = c("estimator_1", "estimator_2")
-#' )
+#' create_data <- function(n) { rpois(n, lambda=5) }
+#' est_mean <- function(dat, type) {
+#'   if (type=="M") { return(mean(dat)) }
+#'   if (type=="V") { return(var(dat)) }
+#' }
+#' sim %<>% set_levels(n=c(10,100,1000), est=c("M","V"))
 #' sim %<>% set_config(num_sim=5)
 #' sim %<>% set_script(function() {
 #'   dat <- create_data(L$n)
-#'   lambda_hat <- use_method(L$estimator, list(dat))
+#'   lambda_hat <- est_mean(dat=dat, type=L$est)
 #'   return (list("lambda_hat"=lambda_hat))
 #' })
 #' sim %<>% run()
@@ -135,10 +134,12 @@ summarize.sim_obj <- function(sim, ...) {
 
   # Error handling
   handle_errors(sim, "is.sim_obj")
-  if (sim$vars$run_state == "pre run")
+  if (sim$vars$run_state == "pre run") {
     stop("Simulation has not been run yet.")
-  if (sim$vars$run_state == "run, all errors")
+  }
+  if (sim$vars$run_state == "run, all errors") {
     stop("100% of simulations had errors.")
+  }
 
   # Parse passed arguments
   R <- sim$results
@@ -181,13 +182,13 @@ summarize.sim_obj <- function(sim, ...) {
                "mse",
                "mae",
                "coverage")
-  for (arg_name in names(o_args)){
-    if (!(arg_name %in% metrics)){
+  for (arg_name in names(o_args)) {
+    if (!(arg_name %in% metrics)) {
       stop(paste0(arg_name, " is an invalid summary metric."))
     }
   }
   for (metric in metrics) {
-    if (!is.null(o_args[[metric]]) && !class(o_args[[metric]][[1]])=="list") {
+    if (!is.null(o_args[[metric]]) && !methods::is(o_args[[metric]][[1]],"list")) {
       o_args[[metric]] <- list(o_args[[metric]])
     }
   }
@@ -206,9 +207,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (m in o_args$mean) {
 
       # if name missing, create a name
-      if (is.null(m$name)){
-        m$name <- paste0("mean_", m$x)
-      }
+      if (is.null(m$name)) { m$name <- paste0("mean_", m$x) }
 
       # Handle errors
       handle_errors(m$x, "is.null", msg="`x` argument is required")
@@ -239,9 +238,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (sd in o_args$sd) {
 
       # if name missing, create a name
-      if (is.null(sd$name)){
-        sd$name <- paste0("sd_", sd$x)
-      }
+      if (is.null(sd$name)) { sd$name <- paste0("sd_", sd$x) }
 
       # Handle errors
       handle_errors(sd$x, "is.null", msg="`x` argument is required")
@@ -272,9 +269,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (var in o_args$var) {
 
       # if name missing, create a name
-      if (is.null(var$name)){
-        var$name <- paste0("var_", var$x)
-      }
+      if (is.null(var$name)) { var$name <- paste0("var_", var$x) }
 
       # Handle errors
       handle_errors(var$x, "is.null", msg="`x` argument is required")
@@ -306,9 +301,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (m in o_args$mad) {
 
       # if name missing, create a name
-      if (is.null(m$name)){
-        m$name <- paste0("MAD_", m$x)
-      }
+      if (is.null(m$name)) { m$name <- paste0("MAD_", m$x) }
 
       # Handle errors
       handle_errors(m$x, "is.null", msg="`x` argument is required")
@@ -340,9 +333,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (i in o_args$iqr) {
 
       # if name missing, create a name
-      if (is.null(i$name)){
-        i$name <- paste0("IQR_", i$x)
-      }
+      if (is.null(i$name)) { i$name <- paste0("IQR_", i$x) }
 
       # Handle errors
       handle_errors(i$x, "is.null", msg="`x` argument is required")
@@ -360,7 +351,7 @@ summarize.sim_obj <- function(sim, ...) {
       code_iqr <- c(code_iqr, paste0(
         #i$name, " = IQR(", i$x, na_1
         pre, i$name, " = tryCatch(IQR(", i$x, na_1,
-        " error = function(e){return(NA)}),"
+        " error = function(e) {return(NA)}),"
       ))
 
     }
@@ -376,9 +367,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (q in o_args$quantile) {
 
       # if name missing, create a name
-      if (is.null(q$name)){
-        q$name <- paste0("quantile_", q$prob, "_", q$x)
-      }
+      if (is.null(q$name)) { q$name <- paste0("quantile_", q$prob, "_", q$x) }
 
       # Handle errors
       handle_errors(q$x, "is.null", msg="`x` argument is required")
@@ -388,7 +377,7 @@ summarize.sim_obj <- function(sim, ...) {
                     msg=paste0("`",q$x,"` is not a variable in results"))
       handle_errors(R[[q$x]], "is.numeric.vec", name=q$x)
       handle_errors(q$prob, "is.numeric", name=q$prob)
-      if (length(q$prob) > 1 | q$prob > 1 | q$prob < 0){
+      if (length(q$prob) > 1 | q$prob > 1 | q$prob < 0) {
         stop(paste0(q$prob, " is not a number between 0 and 1."))
       }
 
@@ -401,7 +390,7 @@ summarize.sim_obj <- function(sim, ...) {
       code_q <- c(code_q, paste0(
         #q$name, " = quantile(", q$x, ", probs=", q$prob, ",", na_1
         pre, q$name, " = tryCatch(quantile(", q$x, ", probs=", q$prob, ",",
-        na_1, " error = function(e){return(NA)}),"
+        na_1, " error = function(e) {return(NA)}),"
       ))
 
     }
@@ -417,9 +406,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (m in o_args$min) {
 
       # if name missing, create a name
-      if (is.null(m$name)){
-        m$name <- paste0("min_", m$x)
-      }
+      if (is.null(m$name)) { m$name <- paste0("min_", m$x) }
 
       # Handle errors
       handle_errors(m$x, "is.null", msg="`x` argument is required")
@@ -450,9 +437,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (m in o_args$max) {
 
       # if name missing, create a name
-      if (is.null(m$name)){
-        m$name <- paste0("max_", m$x)
-      }
+      if (is.null(m$name)) { m$name <- paste0("max_", m$x) }
 
       # Handle errors
       handle_errors(m$x, "is.null", msg="`x` argument is required")
@@ -484,9 +469,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (m in o_args$median) {
 
       # if name missing, create a name
-      if (is.null(m$name)){
-        m$name <- paste0("median_", m$x)
-      }
+      if (is.null(m$name)) { m$name <- paste0("median_", m$x) }
 
       # Handle errors
       handle_errors(m$x, "is.null", msg="`x` argument is required")
@@ -518,9 +501,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (b in o_args$bias) {
 
       # if name missing, create a name
-      if (is.null(b$name)){
-        b$name <- paste0("bias_", b$estimate)
-      }
+      if (is.null(b$name)) { b$name <- paste0("bias_", b$estimate) }
 
       # Handle errors
       handle_errors(b$estimate, "is.null", msg="`estimate` argument is required")
@@ -529,11 +510,9 @@ summarize.sim_obj <- function(sim, ...) {
       handle_errors(b$estimate, "is.in", other=names(R),
                     msg=paste0("`",b$estimate,"` is not a variable in results"))
       handle_errors(R[[b$estimate]], "is.numeric.vec", name=b$estimate)
-      if (
-        length(b$truth)>1 ||
-        (!is.numeric(b$truth) && !(b$truth %in% names(R))) ||
-        (b$truth %in% names(R) && !is.numeric(R[[b$truth]]))
-      ) {
+      if (length(b$truth)>1 ||
+          (!is.numeric(b$truth) && !(b$truth %in% names(R))) ||
+          (b$truth %in% names(R) && !is.numeric(R[[b$truth]]))) {
         stop(paste0("`", b$truth, "` is neither a number nor a variable in results"))
       }
 
@@ -561,9 +540,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (b in o_args$bias_pct) {
 
       # if name missing, create a name
-      if (is.null(b$name)){
-        b$name <- paste0("bias_pct_", b$estimate)
-      }
+      if (is.null(b$name)) { b$name <- paste0("bias_pct_", b$estimate) }
 
       # Handle errors
       handle_errors(b$estimate, "is.null",msg="`estimate` argument is required")
@@ -572,11 +549,9 @@ summarize.sim_obj <- function(sim, ...) {
       handle_errors(b$estimate, "is.in", other=names(R),
                     msg=paste0("`",b$estimate,"` is not a variable in results"))
       handle_errors(R[[b$estimate]], "is.numeric.vec", name=b$estimate)
-      if (
-        length(b$truth)>1 ||
-        (!is.numeric(b$truth) && !(b$truth %in% names(R))) ||
-        (b$truth %in% names(R) && !is.numeric(R[[b$truth]]))
-      ) {
+      if (length(b$truth)>1 ||
+          (!is.numeric(b$truth) && !(b$truth %in% names(R))) ||
+          (b$truth %in% names(R) && !is.numeric(R[[b$truth]]))) {
         stop(paste0("`", b$truth, "` is neither a number nor a variable in results"))
       }
 
@@ -605,9 +580,7 @@ summarize.sim_obj <- function(sim, ...) {
     for (m in o_args$mse) {
 
       # if name missing, create a name
-      if (is.null(m$name)){
-        m$name <- paste0("MSE_", m$estimate)
-      }
+      if (is.null(m$name)) { m$name <- paste0("MSE_", m$estimate) }
 
       # Handle errors
       handle_errors(m$estimate, "is.null", msg="`estimate` argument is required")
@@ -616,11 +589,9 @@ summarize.sim_obj <- function(sim, ...) {
       handle_errors(m$estimate, "is.in", other=names(R),
                     msg=paste0("`",m$estimate,"` is not a variable in results"))
       handle_errors(R[[m$estimate]], "is.numeric.vec", name=m$estimate)
-      if (
-        length(m$truth)>1 ||
-        (!is.numeric(m$truth) && !(m$truth %in% names(R))) ||
-        (m$truth %in% names(R) && !is.numeric(R[[m$truth]]))
-      ) {
+      if (length(m$truth)>1 ||
+          (!is.numeric(m$truth) && !(m$truth %in% names(R))) ||
+          (m$truth %in% names(R) && !is.numeric(R[[m$truth]]))) {
         stop(paste0("`", m$truth, "` is neither a number nor a variable in results"))
       }
 
@@ -642,15 +613,13 @@ summarize.sim_obj <- function(sim, ...) {
 
 
   ### Calculate MAE and parse summary code
-  if(!is.null(o_args[["mae"]])) {
+  if (!is.null(o_args[["mae"]])) {
 
     code_mae <- ""
     for (m in o_args$mae) {
 
       # if name missing, create a name
-      if (is.null(m$name)){
-        m$name <- paste0("MAE_", m$estimate)
-      }
+      if (is.null(m$name)) { m$name <- paste0("MAE_", m$estimate) }
 
       # Handle errors
       handle_errors(m$estimate, "is.null", msg="`estimate` argument is required")
@@ -659,11 +628,9 @@ summarize.sim_obj <- function(sim, ...) {
       handle_errors(m$estimate, "is.in", other=names(R),
                     msg=paste0("`",m$estimate,"` is not a variable in results"))
       handle_errors(R[[m$estimate]], "is.numeric.vec", name=m$estimate)
-      if (
-        length(m$truth)>1 ||
-        (!is.numeric(m$truth) && !(m$truth %in% names(R))) ||
-        (m$truth %in% names(R) && !is.numeric(R[[m$truth]]))
-      ) {
+      if (length(m$truth)>1 ||
+          (!is.numeric(m$truth) && !(m$truth %in% names(R))) ||
+          (m$truth %in% names(R) && !is.numeric(R[[m$truth]]))) {
         stop(paste0("`", m$truth, "` is neither a number nor a variable in results"))
       }
 
@@ -696,11 +663,9 @@ summarize.sim_obj <- function(sim, ...) {
       handle_errors(cov$name, "is.null", msg="`name` argument is required")
       handle_errors(cov$truth, "is.null", msg="`truth` argument is required")
       handle_errors(cov$name, "is.character", name="name")
-      if (
-        length(cov$truth)>1 ||
-        (!is.numeric(cov$truth) && !(cov$truth %in% names(R))) ||
-        (cov$truth %in% names(R) && !is.numeric(R[[cov$truth]]))
-      ) {
+      if (length(cov$truth)>1 ||
+          (!is.numeric(cov$truth) && !(cov$truth %in% names(R))) ||
+          (cov$truth %in% names(R) && !is.numeric(R[[cov$truth]]))) {
         stop(paste0("`", cov$truth, "` is neither a number nor a variable in results"))
       }
       if (!((!is.null(cov$est) && !is.null(cov$se)) ||
